@@ -1,6 +1,6 @@
 package Math::Vector::Real;
 
-our $VERSION = '0.10';
+our $VERSION = '0.14';
 
 use strict;
 use warnings;
@@ -10,8 +10,21 @@ use POSIX ();
 use Exporter qw(import);
 our @EXPORT = qw(V);
 
-local ($@, $!, $SIG{__DIE__});
-eval { require Math::Vector::Real::XS };
+our $dont_use_XS;
+unless ($dont_use_XS) {
+    my $xs_version = do {
+	local ($@, $!, $SIG{__DIE__});
+	eval {
+	    require Math::Vector::Real::XS;
+	    $Math::Vector::Real::XS::VERSION;
+	}
+    };
+
+    if (defined $xs_version and $xs_version < 0.07) {
+	croak "Old and buggy version of Math::Vector::Real::XS detected, update it!";
+    }
+}
+
 
 our %op = (add => '+',
 	   neg => 'neg',
@@ -167,7 +180,7 @@ sub div {
 sub div_me {
     croak "can't use vector as dividend" if ref $_[1];
     my $v = shift;
-    my $i = 1 / shift;
+    my $i = 1.0 / shift;
     $_ *= $i for @$v;
     $v;
 }
@@ -274,11 +287,18 @@ sub _upgrade {
 
 sub atan2 {
     my ($v0, $v1) = @_;
-    my $a0 = &abs($v0);
-    return 0 unless $a0;
-    my $u0 = $v0 / $a0;
-    my $p = $v1 * $u0;
-    CORE::atan2(&abs($v1 - $p * $u0), $p);
+    if (@$v0 == 2) {
+        my $dot = $v0->[0] * $v1->[0] + $v0->[1] * $v1->[1];
+        my $cross = $v0->[0] * $v1->[1] - $v0->[1] * $v1->[0];
+        return CORE::atan2($cross, $dot);
+    }
+    else {
+        my $a0 = &abs($v0);
+        return 0 unless $a0;
+        my $u0 = $v0 / $a0;
+        my $p = $v1 * $u0;
+        CORE::atan2(&abs($v1 - $p * $u0), $p);
+    }
 }
 
 sub versor {
@@ -321,6 +341,21 @@ sub min_component {
 *max = \&max_component;
 *min = \&min_component;
 
+sub first_orthant_reflection {
+    my $self = shift;
+    bless [map CORE::abs, @$self];
+}
+
+sub sum {
+    ref $_[0] or shift; # works both as a class and as an instance method
+    my $sum;
+    if (@_) {
+        $sum = V(@{shift()});
+        $sum += $_ for @_;
+    }
+    return $sum;
+}
+
 sub box {
     shift;
     return unless @_;
@@ -338,7 +373,102 @@ sub box {
             }
         }
     }
-    ($min, $max);
+    wantarray ? ($min, $max) : $max - $min;
+}
+
+sub nearest_in_box {
+    my $p = shift->clone;
+    my ($min, $max) = Math::Vector::Real->box(@_);
+    for (0..$#$p) {
+        if ($p->[$_] < $min->[$_]) {
+            $p->[$_] = $min->[$_];
+        }
+        elsif ($p->[$_] > $max->[$_]) {
+            $p->[$_] = $max->[$_];
+        }
+    }
+    $p
+}
+
+sub dist2_to_box {
+    @_ > 1 or croak 'Usage: $v->dist2_to_box($w0, ...)';
+    my $p = shift;
+    my $d2 = 0;
+    my ($min, $max) = Math::Vector::Real->box(@_);
+    for (0..$#$p) {
+        if ($p->[$_] < $min->[$_]) {
+            my $d = $p->[$_] - $min->[$_];
+            $d2 += $d * $d;
+        }
+        elsif ($p->[$_] > $max->[$_]) {
+            my $d = $p->[$_] - $max->[$_];
+            $d2 += $d * $d;
+        }
+    }
+    $d2;
+}
+
+sub nearest_in_box_border {
+    # TODO: this method can be optimized
+    my $p = shift->clone;
+    my ($b0, $b1) = Math::Vector::Real->box(@_);
+    my $in = 0;
+    for (0..$#$p) {
+        if ($p->[$_] < $b0->[$_]) {
+            $p->[$_] = $b0->[$_];
+        }
+        elsif ($p->[$_] > $b1->[$_]) {
+            $p->[$_] = $b1->[$_];
+        }
+        else {
+            $in++;
+        }
+    }
+    if ($in == @$p) {
+        # vector was inside the box
+        my $min_d = 'inf';
+        my ($comp, $comp_ix);
+        for my $q ($b0, $b1) {
+            for (0..$#$p) {
+                my $d = CORE::abs($p->[$_] - $q->[$_]);
+                if ($min_d > $d) {
+                    $min_d = $d;
+                    $comp = $q->[$_];
+                    $comp_ix = $_;
+                }
+            }
+        }
+        $p->[$comp_ix] = $comp;
+    }
+    $p;
+}
+
+sub max_dist2_to_box {
+    @_ > 1 or croak 'Usage: $v->max_dist2_to_box($w0, ...)';
+    my $p = shift;
+    my ($c0, $c1) = Math::Vector::Real->box(@_);
+    my $d2 = 0;
+    for (0..$#$p) {
+        my $d0 = CORE::abs($c0->[$_] - $p->[$_]);
+        my $d1 = CORE::abs($c1->[$_] - $p->[$_]);
+        $d2 += ($d0 >= $d1 ? $d0 * $d0 : $d1 * $d1);
+    }
+    return $d2;
+}
+
+sub max_dist2_between_boxes {
+    my ($class, $a0, $a1, $b0, $b1) = @_;
+    my ($c0, $c1) = $class->box($a0, $a1);
+    my ($d0, $d1) = $class->box($b0, $b1);
+    my $d2 = 0;
+    for (0..$#$c0) {
+        my $e0 = $d1->[$_] - $c0->[$_];
+        my $e1 = $d0->[$_] - $c1->[$_];
+        $e0 *= $e0;
+        $e1 *= $e1;
+        $d2 += ($e0 > $e1 ? $e0 : $e1);
+    }
+    $d2;
 }
 
 sub max_component_index {
@@ -641,8 +771,45 @@ Returns the distance between the two vectors squared.
 
 =item ($bottom, $top) = Math::Vector::Real->box($v0, $v1, $v2, ...)
 
-Returns the two corners of a hyper-box containing all the given
-vectors.
+Returns the two corners of the L<axis-aligned minimum bounding
+box|http://en.wikipedia.org/wiki/Minimum_bounding_box#Axis-aligned_minimum_bounding_box>
+(or L<hyperrectangle|http://en.wikipedia.org/wiki/Hyperrectangle>) for
+the given vectors.
+
+In scalar context returns the difference between the two corners (the
+box diagonal vector).
+
+=item $p = $v->nearest_in_box($w0, $w1, ...)
+
+Returns the vector nearest to C<$v> from the axis-aligned minimum box
+bounding the given set of vectors.
+
+For instance, given a point C<$v> and an axis-aligned rectangle
+defined by two opposite corners (C<$c0> and C<$c1>), this method can be
+used to find the point nearest to C<$v> from inside the rectangle:
+
+  my $n = $v->nearest_in_box($c0, $c1);
+
+Note that if C<$v> lays inside the box, the nearest point is C<$v>
+itself. Otherwise it will be a point from the box hyper-surface.
+
+=item $d2 = $v->dist2_to_box($w0, $w1, ...)
+
+Calculates the square of the minimal distance between the vector C<$v>
+and the minimal axis-aligned box containing all the vectors C<($w0,
+$w1, ...)>.
+
+=item $d2 = $v->max_dist2_to_box($w0, $w1, ...)
+
+Calculates the square of the maximum distance between the vector C<$v>
+and the minimal axis-aligned box containing all the vectors C<($w0,
+$w1, ...)>.
+
+=item $d2 = Math::Vector::Real->max_dist2_between_boxes($a0, $a1, $b0, $b1)
+
+Returns the square of the maximum distance between any two points
+belonging respectively to the boxes defined by C<($a0, $a1)> and
+C<($b0, $b1)>.
 
 =item $v->set($u)
 
@@ -652,7 +819,17 @@ Note that this method is destructive.
 
 =item $d = $v->max_component_index
 
-Return the index of the vector component with the maximum size.
+Returns the index of the vector component with the maximum size.
+
+=item $r = $v->first_orthant_reflection
+
+Given the set of vectors formed by C<$v> and all its reflections
+around the axis-aligned hyperplanes, this method returns the one lying
+on the first orthant.
+
+See also
+[http://en.wikipedia.org/wiki/Reflection_%28mathematics%29|reflection]
+and [http://en.wikipedia.org/wiki/Orthant|orthant].
 
 =item ($p, $n) = $v->decompose($u)
 
@@ -661,6 +838,10 @@ and another normal.
 
 In scalar context returns the normal vector.
 
+=item $v = Math::Vector::Real->sum(@v)
+
+Returns the sum of all the given vectors.
+
 =item @b = Math::Vector::Real->complementary_base(@v)
 
 Returns a base for the subspace complementary to the one defined by
@@ -672,7 +853,7 @@ wrong result may be generated.
 
 =item @b = $v->normal_base
 
-Returns a set of vectors forming an ortonormal base for the hyperplane
+Returns a set of vectors forming an orthonormal base for the hyperplane
 normal to $v.
 
 In scalar context returns just some unitary vector normal to $v.
@@ -727,6 +908,12 @@ checks of this kind:
 Or even better, reorder the operations to minimize the chance of
 instabilities if the algorithm allows it.
 
+=head2 Math::Vector::Real::XS
+
+The module L<Math::Vector::Real::XS> reimplements most of the methods
+available from this module in XS. When it is installed,
+C<Math::Vector::Real> when automatically load and use it.
+
 =head1 SEE ALSO
 
 L<Math::Vector::Real::Random> extends this module with random vector
@@ -738,9 +925,23 @@ There are other vector manipulation packages in CPAN (L<Math::Vec>,
 L<Math::VectorReal>, L<Math::Vector>), but they can only handle 3
 dimensional vectors.
 
+=head1 SUPPORT
+
+In order to report bugs you can send me and email to the address that
+appears below or use the CPAN RT bug-tracking system available at
+L<http://rt.cpan.org>.
+
+The source for the development version of the module is hosted at
+GitHub: L<https://github.com/salva/p5-Math-Vector-Real>.
+
+=head2 My wishlist
+
+If you like this module and you're feeling generous, take a look at my
+wishlist: L<http://amzn.com/w/1WU1P6IR5QZ42>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2009-2012 by Salvador FandiE<ntilde>o
+Copyright (C) 2009-2012, 2014 by Salvador FandiE<ntilde>o
 (sfandino@yahoo.com)
 
 This library is free software; you can redistribute it and/or modify
